@@ -2,7 +2,7 @@
 #
 # django-codenerix-pos
 #
-# Copyright 2017 Centrologic Computational Logistic Center S.L.
+# Codenerix GNU
 #
 # Project URL : http://www.codenerix.com
 #
@@ -20,11 +20,12 @@
 
 from django.contrib import admin
 
-from .models import POS, POSSlot, POSPlant, POSZone, POSProduct, POSHardware, POSOperator, POSLog
+from .models import POS, POSSlot, POSPlant, POSZone, POSProduct, POSHardware, POSOperator, POSLog, POSGroupProduct
 
 admin.site.register(POSPlant)
 admin.site.register(POSZone)
 admin.site.register(POSHardware)
+admin.site.register(POSGroupProduct)
 admin.site.register(POS)
 admin.site.register(POSSlot)
 admin.site.register(POSProduct)
